/*
 * Dine-O-Matic v2.0
 * Copyright 2007 - Iconfactory - http://iconfactory.com
 * 
 * Programmer: Joe Roback <joe.roback@gmail.com>
 * Graphics: Talos Tsui, Corey Marion, David Lanham
 * Audio: Dave Brasgalla
 *
 */

function formatPixelLength(len) {
	if (len == null) {
		return null;
	}
	else if (len == 'auto') {
		return 'auto';
	}
	else {
		return len + 'px';
	}
}

function DOMButton(button, width, height, img, imgClicked, imgDisabled, imgHover, onclick, controlRegion)
{
	if (button == null || img == null || imgClicked == null) {
		return;
	}
	
	if (!(controlRegion == 'rectangle' || controlRegion == 'circle')) {
		return;
	}
	
	/* Read-write properties */
	this.onclick = onclick;
	
	/* Read-only properties */
	this.enabled = true;
	
	this._init(
		button,
		null,
		null,
		width,
		height,
		null,
		img,
		null,
		null,
		imgClicked,
		null,
		null,
		imgDisabled,
		null,
		null,
		imgHover,
		null,
		null,
		onclick,
		controlRegion);
}

DOMButton.prototype._init = function(
	button,
	top,
	left,
	width,
	height,
	text,
	img,
	imgLeft,
	imgRight,
	imgClicked,
	imgClickedLeft,
	imgClickedRight,
	imgDisabled,
	imgDisabledLeft,
	imgDisabledRight,
	imgHover,
	imgHoverLeft,
	imgHoverRight,
	onclick,
	controlRegion)
{
	var fWidth = formatPixelLength(width);
	var fHeight = formatPixelLength(height);

	this._imgPath = img;
	this._imgLeftPath = imgLeft;
	this._imgRightPath = imgRight;
	
	this._imgClickedPath = imgClicked;
	this._imgClickedLeftPath = imgClickedLeft;
	this._imgClickedRightPath = imgClickedRight;
	
	this._imgDisabledPath = imgDisabled;
	this._imgDisabledLeftPath = imgDisabledLeft;
	this._imgDisabledRightPath = imgDisabledRight;
	
	this._imgHoverPath = imgHover;
	this._imgHoverLeftPath = imgHoverLeft;
	this._imgHoverRightPath = imgHoverRight;

	// preload clicked images
	var tmpImg = new Image();
	tmpImg.src = this._imgPath;
	tmpImg = new Image();
	tmpImg.src = this._imgLeftPath;
	tmpImg = new Image();
	tmpImg.src = this._imgRightPath;
	tmpImg = new Image();
	tmpImg.src = this._imgClickedPath;
	tmpImg = new Image();
	tmpImg.src = this._imgClickedLeftPath;
	tmpImg = new Image();
	tmpImg.src = this._imgClickedRightPath;
	tmpImg = new Image();
	tmpImg.src = this._imgDisabledPath;
	tmpImg = new Image();
	tmpImg.src = this._imgDisabledLeftPath;
	tmpImg = new Image();
	tmpImg.src = this._imgDisabledRightPath;
	tmpImg = new Image();
	tmpImg.src = this._imgHoverPath;
	tmpImg = new Image();
	tmpImg.src = this._imgHoverLeftPath;
	tmpImg = new Image();
	tmpImg.src = this._imgHoverRightPath;
	
	button.innerText = null;
	var style = button.style;
	//style.border = '1px solid blue';
	if (top) {
		style.top = formatPixelLength(top);
	}
	if (left) {
		style.left = formatPixelLength(left);
	}
	//alert(button.outerHTML);
	
	this._container = document.createElement('div');
	style = this._container.style;
	//style.border = '1px solid yellow';
	style.display = 'block';
	style.width = formatPixelLength(parseInt(width) + 4);
	style.height = fHeight;
	style.appleDashboardRegion = 'dashboard-region(control ' + controlRegion + ' 0px 0px 0px 0px)';

	if (this._imgLeftPath) {
		this._left = document.createElement('div');
		style = this._left.style;
		style.width = '2px';
		style.height = fHeight
		style.margin = '0px 0px 0px 0px';
		style.padding = '0px 0px 0px 0px';
		style.background = 'url(' + this._imgLeftPath + ') no-repeat top left';
		style.display = 'inline-block';
		this._container.appendChild(this._left);
	}
	
	this._middle = document.createElement('div');
	style = this._middle.style;
	style.margin = '0px 0px 0px 0px';
	style.padding = '0px 0px 0px 0px';
	style.height = fHeight;
	style.width = fWidth;

	if (text == null) {
		style.backgroundRepeat = 'no-repeat';
	}
	else {
		style.backgroundRepeat = 'repeat-x';
	}
	style.backgroundImage = 'url(' + this._imgPath + ')';
	style.backgroundPosition = 'top left';
	style.display = 'inline-block';
	this._container.appendChild(this._middle);
	
	// inner text
	if (text != null) {
		this._text = document.createElement('div');
		this._text.innerText = text;
		style = this._text.style;
		style.textAlign = 'center';
		style.width = 'auto';
		style.height = fHeight;
		style.lineHeight = fHeight;
		style.margin = '1px 0px 0px 0px';
		style.padding = '0px 2px 0px 2px';
		this._middle.appendChild(this._text);
	}
	
	if (this._imgRightPath) {
		this._right = document.createElement('div');
		style = this._right.style;
		style.width = '2px';
		style.height = fHeight;
		style.margin = '0px 0px 0px 0px';
		style.padding = '0px 0px 0px 0px';
		style.background = 'url(' + this._imgRightPath + ') no-repeat top left';
		style.display = 'inline-block';
		this._container.appendChild(this._right);
	}
	
	// For JavaScript event handlers
	var _self = this;
	this._mousedownHandler = function(event) { _self._mousedown(event); }
	this._mousemoveHandler = function(event) { _self._mousemove(event); }
	this._mouseoutHandler = function(event) { _self._mouseout(event); }
	this._mouseoverHandler = function(event) { _self._mouseover(event); }
	this._mouseupHandler = function(event) { _self._mouseup(event); }

	this._mouseoutHoverHandler = function(event) { _self._mouseoutHover(event); }
	this._mouseoverHoverHandler = function(event) { _self._mouseoverHover(event); }
	
	this._addEventListeners();
	button.appendChild(this._container);
}

DOMButton.prototype._addEventListeners = function()
{
	this._container.addEventListener('mousedown', this._mousedownHandler, true);
	if (this._imgHoverPath !== undefined && this._imgHoverPath != null) {
		this._container.addEventListener('mouseover', this._mouseoverHoverHandler, true);
		this._container.addEventListener('mouseout', this._mouseoutHoverHandler, true);
	}
}

DOMButton.prototype._removeEventListeners = function()
{
	this._container.removeEventListener('mousedown', this._mousedownHandler, true);
	if (this._imgHoverPath !== undefined && this._imgHoverPath != null) {
		this._container.removeEventListener('mouseover', this._mouseoverHoverHandler, true);
		this._container.removeEventListener('mouseout', this._mouseoutHoverHandler, true);
	}
}

DOMButton.prototype.remove = function()
{
	var parent = this._container.parentNode;
	parent.removeChild(this._container);
}

DOMButton.prototype.setEnabled = function(enabled)
{
	this.enabled = enabled;
	
	if (enabled) {
		this._middle.style.backgroundImage = 'url(' + this._imgPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgClickedLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgClickedRightPath + ')';
		}
		
		this._container.style.appleDashboardRegion = 'dashboard-region(control rectangle 0px 0px 0px 0px)';
		this._addEventListeners();
	}
	else if (this._imgDisabledPath !== undefined) {
		this._removeEventListeners();
		this._container.style.appleDashboardRegion = 'none';
		this._middle.style.backgroundImage = 'url(' + this._imgDisabledPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgDisabledLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgDisabledRightPath + ')';
		}
	}
}

/*********************
* Private handlers
*/

DOMButton.prototype._setPressed = function(pressed)
{
	if (pressed) {
		if (this._text) { this._text.style.marginTop = '2px'; }
		this._middle.style.backgroundImage = 'url(' + this._imgClickedPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgClickedLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgClickedRightPath + ')';
		}
	}
	else {
		if (this._text) { this._text.style.marginTop = '1px'; }
		this._middle.style.backgroundImage = 'url(' + this._imgPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgRightPath + ')';
		}
	}
}

DOMButton.prototype._setHover = function(inside)
{
	if (inside) {
		this._middle.style.backgroundImage = 'url(' + this._imgHoverPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgHoverLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgHoverRightPath + ')';
		}
	}
	else {
		this._middle.style.backgroundImage = 'url(' + this._imgPath + ')';
		if (this._left) {
			this._left.style.backgroundImage = 'url(' + this._imgLeftPath + ')';
		}
		if (this._right) {
			this._right.style.backgroundImage = 'url(' + this._imgRightPath + ')';
		}
	}
}

DOMButton.prototype._mousedown = function(event)
{
	// If we're disabled, don't do anything
	if (!this.enabled) {
		event.stopPropagation();
		event.preventDefault();
		
		return;
	}
	
	this._mouseoutBackgroundImage = this._middle.style.backgroundImage;
	if (this._left) { this._mouseoutLeftBackgroundImage = this._left.style.backgroundImage; }
	if (this._right) { this._mouseoutRightBackgroundImage = this._right.style.backgroundImage; }
	
	// Change images to clicked state
	this._setPressed(true);
	
	this._mouseoverBackgroundImage = this._middle.style.backgroundImage;
	if (this._left) { this._mouseoverLeftBackgroundImage = this._left.style.backgroundImage; }
	if (this._right) { this._mouseoverRightBackgroundImage = this._right.style.backgroundImage; }
	
	// add event listeners
	document.addEventListener('mousemove', this._mousemoveHandler, true);
	document.addEventListener('mouseup', this._mouseupHandler, true);
	this._container.addEventListener('mouseover', this._mouseoverHandler, true);
	this._container.addEventListener('mouseout', this._mouseoutHandler, true);
	
	this._down = true;
	this._inside = true;
	
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._mouseup = function(event)
{
	this._setPressed(false);
	
	// Remove event listeners
	this._removeInternalMouseHandlers();
	
	try {
		if (this._inside && this.onclick != null) {
			this.onclick(event);
		}
	}
	catch (ex) {
		throw ex;
	}
	finally {
		event.stopPropagation();
		event.preventDefault();
		
		this._mouseupCleanup();
	}
}

DOMButton.prototype._mouseover = function(event)
{
	if (this._text) { this._text.style.marginTop = '2px'; }
	this._middle.style.backgroundImage = this._mouseoverBackgroundImage;
	if (this._left) { this._left.style.backgroundImage = this._mouseoverLeftBackgroundImage; }
	if (this._right) { this._right.style.backgroundImage = this._mouseoverRightBackgroundImage; }
	
	this._inside = true;
	
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._mouseout = function(event)
{
	if (this._text) { this._text.style.marginTop = '1px'; }
	this._middle.style.backgroundImage = this._mouseoutBackgroundImage;	
	if (this._left) { this._left.style.backgroundImage = this._mouseoutLeftBackgroundImage; }
	if (this._right) { this._right.style.backgroundImage = this._mouseoutRightBackgroundImage; }

	this._inside = false;
	
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._mousemove = function(event)
{
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._mouseoverHover = function(event)
{
	if (this._down == undefined || this._down == false) {
		this._setHover(true);
	}
	
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._mouseoutHover = function(event)
{
	if (this._down == undefined || this._down == false) {
		this._setHover(false);
	}
	
	event.stopPropagation();
	event.preventDefault();
}

DOMButton.prototype._removeInternalMouseHandlers = function()
{
	document.removeEventListener('mousemove', this._mousemoveHandler, true);
	document.removeEventListener('mouseup', this._mouseupHandler, true);
	this._container.removeEventListener('mouseover', this._mouseoverHandler, true);
	this._container.removeEventListener('mouseout', this._mouseoutHandler, true);
}

DOMButton.prototype._mouseupCleanup = function()
{
	delete this._inside;
	delete this._down;
	delete this._mouseoverBackgroundImage;
	delete this._mouseoverLeftBackgroundImage;
	delete this._mouseoverRightBackgroundImage;
	delete this._mouseoutBackgroundImage;
	delete this._mouseoutLeftBackgroundImage;
	delete this._mouseoutRightBackgroundImage;
}

//
// ToogleButton
//

function DOMToggleButton(
	button,
	top,
	left,
	width,
	height,
	text,
	img,
	imgLeft,
	imgRight,
	imgClicked,
	imgClickedLeft,
	imgClickedRight,
	imgDisabled,
	imgDisabledLeft,
	imgDisabledRight,
	imgHover,
	imgHoverLeft,
	imgHoverRight,
	imgToggled,
	imgToggledLeft,
	imgToggledRight,
	onclick,
	controlRegion)
{
	this.toggled = false;
	
	this.onclick = null;
	this.toggle_onclick = onclick;

	this._imgToggledPath = imgToggled;
	var tmpImg = new Image();
	tmpImg.src = this._imgToggledPath;

	this._imgToggledLeftPath = imgToggledLeft;
	var tmpImg = new Image();
	tmpImg.src = this._imgToggledLeftPath;

	this._imgToggledRightPath = imgToggledRight;
	var tmpImg = new Image();
	tmpImg.src = this._imgToggledRightPath;
	
	this._init(
		button,
		top,
		left,
		width,
		height,
		text,
		img,
		imgLeft,
		imgRight,
		imgClicked,
		imgClickedLeft,
		imgClickedRight,
		imgDisabled,
		imgDisabledLeft,
		imgDisabledRight,
		imgHover,
		imgHoverLeft,
		imgHoverRight,
		onclick,
		controlRegion);
}

DOMToggleButton.prototype = new DOMButton(null);

DOMToggleButton.prototype.setEnabled = function(enabled)
{
	DOMButton.prototype.setEnabled.call(this, enabled);
	
	if (enabled) {
		this._container.addEventListener('click', this._clickHandler, true);
	}
	else {
		this._container.removeEventListener('click', this._clickHandler, true);
	}
}

DOMToggleButton.prototype.setToggled = function(toggled)
{
	this.toggled = toggled;
	
	if (toggled) {
		this._middle.style.backgroundImage = 'url(' + this._imgToggledPath + ')';
	}
	else {
		this._setPressed(false);
	}
}

DOMToggleButton.prototype._mouseup = function(event)
{
	if (this._inside) {
		this.toggled = !this.toggled;
	}
	
	// Remove event listeners
	this._removeInternalMouseHandlers();
	
	try {
		if (this.toggled) {
			this._middle.style.backgroundImage = 'url(' + this._imgToggledPath + ')';
		}
		else {
			this._setPressed(false);
		}
		
		if (this._inside && this.toggle_onclick != null) {
			this.toggle_onclick(event, this.toggled);
		}
	}
	catch (ex) {
		throw ex;
	}
	finally {
		event.stopPropagation();
		event.preventDefault();

		this._mouseupCleanup();
	}
}

//
// TextButton
//

function DOMTextButton(
	button,
	top,
	left,
	width,
	height,
	text,
	img,
	imgLeft,
	imgRight,
	imgClicked,
	imgClickedLeft,
	imgClickedRight,
	imgDisabled,
	imgDisabledLeft,
	imgDisabledRight,
	imgHover,
	imgHoverLeft,
	imgHoverRight,
	onclick,
	controlRegion)
{
	if (button == null || img == null || imgClicked == null) {
		return;
	}
	
	if (!(controlRegion == 'rectangle' || controlRegion == 'circle')) {
		return;
	}
	
	/* Read-write properties */
	this.onclick = onclick;
	
	/* Read-only properties */
	this.enabled = true;

	this._init(
		button,
		top,
		left,
		width,
		height,
		text,
		img,
		imgLeft,
		imgRight,
		imgClicked,
		imgClickedLeft,
		imgClickedRight,
		imgDisabled,
		imgDisabledLeft,
		imgDisabledRight,
		imgHover,
		imgHoverLeft,
		imgHoverRight,
		onclick,
		controlRegion);
}

DOMTextButton.prototype = new DOMButton(null);
