/*
 * Dine-O-Matic v2.0
 * Copyright 2007 - Iconfactory - http://iconfactory.com
 * 
 * Programmer: Joe Roback <joe.roback@gmail.com>
 * Graphics: Talos Tsui, Corey Marion, David Lanham
 * Audio: Dave Brasgalla
 *
 */

function DOMListView(listviewElement, spec)
{
	this._init(listviewElement, spec);
}

DOMListView.prototype._init = function(listviewElement, spec)
{
	var scrollbarSize = 19;
	
	this._selectedItem = { object: null, className: null };
	this._listviewElement = listviewElement;
	
	this._rowClass = spec.rowClass;
	this._selectedRowClass = spec.selectedRowClass;
	this._onselect = spec.onselect;
	this._onselectionclear = spec.onselectionclear;
	
	var content = document.createElement('div');
	style = content.style;
	style.position = 'absolute';
	style.backgroundImage = spec.backgroundImage;
	style.backgroundPosition = spec.backgroundPosition;
	style.backgroundRepeat = spec.backgroundRepeat;
	style.left = '0px';
	style.top = '0px';
	style.right = scrollbarSize + 'px';
	style.bottom = '0px';
	
	listviewElement.appendChild(content);
	listviewElement.content = content;
	
	var scrollbarElement = document.createElement('div');
	scrollbarElement.className = 'apple-no-children apple-remove';
	style = scrollbarElement.style;
	style.position = 'absolute';
	style.width = scrollbarSize + 'px';
	style.height = 'auto';
	style.right = '0px';
	style.top = '2px';
	style.bottom = '2px';
	style.appleDashboardRegion = 'none';
	
	listviewElement.appendChild(scrollbarElement);
	
	verticalScrollbar = new AppleVerticalScrollbar(scrollbarElement);
	listviewElement.verticalScrollbarElement = scrollbarElement;
	
	listviewElement.object = new AppleScrollArea(content, verticalScrollbar);
	listviewElement.object.content = content;
	listviewElement.object.refresh();
	
	verticalScrollbar.setAutohide(false);
	
	// events, argh!
	var _self = this;
	this._rowClickHandler = function(event) { _self._row_onclick(event); }
}

DOMListView.prototype.setEnabled = function(enabled)
{
	var node = this._listviewElement.content;
	var subNode = null;
	
	// hacks, hacks and more hacks
	// use previouslyDisabled to remember the disable state
	// if enabled == false, record old disabled
	// then on next enable == true store that state, instead
	// of blindly enabling everything...
	
	for (node = node.firstChild; node != null; node = node.nextSibling) {
		if (!enabled) {
			node.previouslyDisabled = node.disabled;
		}
		
		node.previouslyDisabled = node.previouslyDisabled == undefined ? false : node.previouslyDisabled;
		node.disabled = enabled ? node.previouslyDisabled : true;
		
		for (subNode = node.firstChild; subNode != null; subNode = subNode.nextSibiling) {
			if (subNode.tagName.toLowerCase() == 'input') {
				if (!enabled) {
					subNode.previouslyDisabled = subNode.disabled;
				}
				
				subNode.previouslyDisabled = subNode.previouslyDisabled == undefined ? false : subNode.previouslyDisabled;
				subNode.disabled = enabled ? subNode.previouslyDisabled : true;
			}
		}
	}
}

DOMListView.prototype.refresh = function()
{
	this._listviewElement.object.refresh();
}

DOMListView.prototype.reveal = function(obj)
{
	if (obj != null) {
		this._listviewElement.object.reveal(obj);
	}
}

DOMListView.prototype.clear = function()
{
	this.clearSelection();
	
	this._listviewElement.content.innerHTML = '';
	this._listviewElement.object.refresh();
}

DOMListView.prototype.append = function(columns)
{
	var row = this._createRowDiv(columns);
	
	// append row to end of listview
	this._listviewElement.content.appendChild(row);
	
	return row;
}

DOMListView.prototype.insert = function(columns, index)
{
	var row = this._createRowDiv(columns);
	var node = this._listviewElement.content;
	
	// insert row at index
	node.insertBefore(row, node.childNodes[index]);
	
	return row;
}

DOMListView.prototype.remove = function(row)
{
	this._listviewElement.content.removeChild(row);
	
	if (this._selectedItem.object == row) {
		this._clearSelection(false);
	}
}

DOMListView.prototype.removeSelected = function()
{
	this._listviewElement.content.removeChild(this._selectedItem.object);
	this._clearSelection(false);
}

DOMListView.prototype.select = function(obj)
{
	if (obj != null) {
		this._selectRow(obj, false);
	}
}

DOMListView.prototype.selectIndex = function(index)
{
	if (index >= 0 && index < this._listviewElement.content.childNodes.length) {
		this._selectRow(this._listviewElement.content.childNodes[index], false);
	}
	
	return this._selectedItem.object;
}

DOMListView.prototype.revealSelected = function()
{
	if (this._selectedItem.object != null) {
		this._listviewElement.object.reveal(this._selectedItem.object);
	}
}

DOMListView.prototype.selectAndReveal = function(obj)
{
	if (obj != null) {
		this._selectRow(obj, false);
		this._listviewElement.object.reveal(obj);
	}
}

DOMListView.prototype.selected = function()
{
	return this._selectedItem.object;
}

DOMListView.prototype.selectedIndex = function()
{
	if (this._selectedItem.object != null) {
		return this.indexOf(this._selectedItem.object);
	}
	
	return -1;
}

DOMListView.prototype.clearSelection = function()
{
	this._clearSelection(true);
}

DOMListView.prototype._clearSelection = function(fire_event)
{
	if (this._selectedItem.object) {
		this._selectedItem.object.className = this._selectedItem.className;
		this._selectedItem.object = null;
		this._selectedItem.className = null;
		
		// invoke user defined handler
		if (fire_event && this._onselectionclear != null) {
			this._onselectionclear();
		}
	}
}

DOMListView.prototype.indexOf = function(obj)
{
	var node = this._listviewElement.content;
	var i = 0;
	
	for (node = node.firstChild; node != null; node = node.nextSibling) {
		if (node == obj) {
			return i;
		}
		i++;
	}
}

/* PRIVATE */

DOMListView.prototype._createRowDiv = function(columns)
{
	var row = document.createElement('div');
	row.className = this._rowClass;
	row.disabled = false;
	row.onselect = this._onselect;
	row.addEventListener('click', this._rowClickHandler, true);
	
	// append columns, each element should be a DOM object
	for (var i = 0; i < columns.length; i++) {
		row.appendChild(columns[i]);
	}
	
	return row;
}

DOMListView.prototype._selectRow = function(row, fire_event_handler)
{
	var oldrow = null;
	
	if (!row.disabled) {
		if (this._selectedItem.object) {
			//oldrow = this._selectedItem.object;
			this._selectedItem.object.className = this._selectedItem.className;
		}
		
		this._selectedItem.object = row;
		this._selectedItem.className = row.className;
		
		row.className += ' ' + this._selectedRowClass;
		
		// invoke user event handler
		if (row.onselect != null && fire_event_handler) {
			row.onselect(row, oldrow);
		}
	}
}

DOMListView.prototype._row_onclick = function(event)
{
	var row = event.currentTarget;
	
	try {
		this._selectRow(row, true);
	}
	catch (ex) {
		throw ex;
	}
	finally {	
		if (event) {
			event.stopPropagation();
			event.preventDefault();
		}
	}
}
