/*
 * Dine-O-Matic v2.0
 * Copyright 2007 - Iconfactory - http://iconfactory.com
 * 
 * Programmer: Joe Roback <joe.roback@gmail.com>
 * Graphics: Talos Tsui, Corey Marion, David Lanham
 * Audio: Dave Brasgalla
 *
 */

function DOMNoticeBox(boxElement)
{
	this._init(boxElement, {});
}

DOMNoticeBox.prototype._init = function(boxElement, spec)
{
	var style = null;
	
	var width = 215;
	var height = 39;
	var tHeight = 20;
	var mHeight = 3;
	var bHeight = 31;
	
	// preload images
	var tImg = new Image(width, tHeight);
	tImg.src = 'Images/noticebox_top.png';
	var mImg = new Image(width, mHeight);
	mImg.src = 'Images/noticebox_middle.png';
	var bImg = new Image(width, bHeight);
	bImg.src = 'Images/noticebox_bottom.png';
	
	// default to hide
	this.hidden = true;
	
	this._boxElement = boxElement;
	style = this._boxElement.style;
	style.display = 'none';
	style.opacity = 0;
	
	this._container = document.createElement('div');
	style = this._container.style;
	style.width = width + 'px';
	style.margin = '0px';
	style.padding = '0px';
	
	var topbox = document.createElement('div');
	style = topbox.style;
	style.width = width + 'px';
	style.height = tHeight + 'px';
	style.backgroundImage = 'url(' + tImg.src + ')';
	style.backgroundRepeat = 'no-repeat';
	style.backgroundPosition = 'top left';
	this._container.appendChild(topbox);
	
	var middlebox = document.createElement('div');
	style = middlebox.style;
	style.width = width + 'px';
	style.height = height + 'px';
	style.backgroundImage = 'url(' + mImg.src + ')';
	style.backgroundRepeat = 'repeat-y';
	style.backgroundPosition = 'top left';
	style.overflow = 'hidden';
	this._container.appendChild(middlebox);
	
	this._content = document.createElement('div');
	style = this._content.style;
	style.padding = '2px 14px';
	style.color = 'rgb(102, 102, 102)';
	style.fontFamily = 'Helvetica, sans-serif';
	style.fontSize = '11px';
	style.lineHeight = '12px';
	style.textAlign = 'left';
	style.verticalAlign = 'top';
	middlebox.appendChild(this._content);

	var bottombox = document.createElement('div');
	style = bottombox.style;
	style.width = width + 'px';
	style.height = bHeight + 'px';
	style.backgroundImage = 'url(' + bImg.src + ')';
	style.backgroundRepeat = 'no-repeat';
	style.backgroundPosition = 'top left';
	this._container.appendChild(bottombox);
	
	var _self = this;
	this._mousedownHandler = function(event) { _self._mousedown(event); }
		
	this._boxElement.appendChild(this._container);
}

DOMNoticeBox.prototype.remove = function()
{
	var parent = this._container.parentNode;
	parent.removeChild(this._container);
}

DOMNoticeBox.prototype.setContentHTML = function(content)
{
	this._content.innerHTML = content;
}

DOMNoticeBox.prototype.setContentText = function(content)
{
	this._content.innerTextL = content;
}

DOMNoticeBox.prototype.show = function(fade)
{
	if (this.hidden == false) {
		return;
	}
	
	this.hidden = false;
	
	if (fade) {
		var fader = new AppleAnimator(375, 33, 0.0, 1.0, this._fade);
		fader._domNoticeBox = this;
		fader.start();
	}
	else {
		this._show_private();
	}
}

DOMNoticeBox.prototype.hide = function(fade)
{
	if (this.hidden == true) {
		return;
	}

	this.hidden = true;
	
	if (fade) {
		var fader = new AppleAnimator(350, 50, 1.0, 0.0, this._fade);
		fader._domNoticeBox = this;
		fader.start();
	}
	else {
		this._hide_private();
	}
}

DOMNoticeBox.prototype._show_private = function()
{
	this._boxElement.style.display = 'block';
	this._boxElement.style.opacity = 1;
	this._boxElement.addEventListener('mousedown', this._mousedownHandler, true);
}

DOMNoticeBox.prototype._hide_private = function()
{
	this._boxElement.removeEventListener('mousedown', this._mousedownHandler, true);
	this._boxElement.style.display = 'none';
	this._boxElement.style.opacity = 0;
}

DOMNoticeBox.prototype._fade = function(animation, current, start, finish)
{
	if (start && animation._domNoticeBox.hidden == false) {
		animation._domNoticeBox._show_private();
	}
	else if (finish && animation._domNoticeBox.hidden == true) {
		animation._domNoticeBox._hide_private();
	}
	
	animation._domNoticeBox._boxElement.style.opacity = current;
}

DOMNoticeBox.prototype._mousedown = function(event)
{
	this.hide(true);
}
