/*
 * Dine-O-Matic v2.0
 * Copyright 2007 - Iconfactory - http://iconfactory.com
 * 
 * Programmer: Joe Roback <joe.roback@gmail.com>
 * Graphics: Talos Tsui, Corey Marion, David Lanham
 * Audio: Dave Brasgalla
 *
 */

function DOMPopup(popupElement,
	htmlValues,
	textValues,
	values,
	height,
	leftImage,
	leftImageWidth,
	middleImage,
	middleImageWidth,
	middleClass,
	rightImage,
	rightImageWidth,
	onchange)
{
	if (popupElement == null || leftImage == null || middleImage == null || rightImage == null) {
		return;
	}
	
	if (htmlValues.length != textValues.length || htmlValues.length != values.length) {
		return;
	}
	
	this.onchange = onchange;
	this.value = null;
	this.style = popupElement.style;
	
	this._init(popupElement, htmlValues, textValues, values, height, leftImage, leftImageWidth, middleImage, middleImageWidth, middleClass, rightImage, rightImageWidth);
}

DOMPopup.prototype._init = function(popupElement, htmlValues, textValues, values, height, leftImage, leftImageWidth, middleImage, middleImageWidth, middleClass, rightImage, rightImageWidth)
{
	var style = null;
	
	this._popupElement = popupElement;
	this._width = leftImageWidth + middleImageWidth + rightImageWidth;
	this._height = height;
	
	// preload images
	this._leftImage = new Image(leftImageWidth, height);
	this._leftImage.src = leftImage;
	this._middleImage = new Image(middleImageWidth, height);
	this._middleImage.src = middleImage;
	this._rightImage = new Image(rightImageWidth, height);
	this._rightImage.src = rightImage;
	
	style = this._popupElement.style;
	style.cssFloat = 'none';
	
	this._leftDiv = document.createElement('div');
	style = this._leftDiv.style;
	style.position = 'absolute';
	style.top = '0px';
	style.left = '0px';
	style.margin = '0px';
	style.padding = '0px';
	style.width = leftImageWidth + 'px';
	style.height = height + 'px';
	style.background = 'transparent url(' + this._leftImage.src + ') no-repeat top left';
	
	this._middleDiv = document.createElement('div');
	this._middleDiv.className = middleClass;
	this._middleDiv.innerHTML = htmlValues[0];
	style = this._middleDiv.style;
	style.position = 'absolute';
	style.top = '0px';
	style.left = leftImageWidth + 'px';
	style.margin = '0px';
	style.padding = '0px';
	style.width = middleImageWidth + 'px';
	style.height = height + 'px';
	style.background = 'transparent url(' + this._middleImage.src + ') repeat-x top left';
	style.overflow = 'hidden';
	style.lineHeight = (parseInt(height) + 1) + 'px';
	
	this._rightDiv = document.createElement('div');
	style = this._rightDiv.style;
	style.position = 'absolute';
	style.top = '0px';
	style.left = (leftImageWidth + middleImageWidth) + 'px';
	style.margin = '0px';
	style.padding = '0px';
	style.width = rightImageWidth + 'px';
	style.height = height + 'px';
	style.background = 'transparent url(' + this._rightImage.src + ') no-repeat top left';
	
	this._select = document.createElement('select');
	style = this._select.style;
	style.position = 'absolute';
	style.top = '0px';
	style.left = '0px';
	style.margin = '0px';
	style.padding = '0px';
	style.width = this._width + 'px';
	style.height = height + 'px';
	style.opacity = 0.0;
	
	// event handlers
	var _self = this;
	this._onchangeHandler = function(event) { _self._onchange(event); }
	this._select.addEventListener("change", this._onchangeHandler, true);
	
	this.setValues(htmlValues, textValues, values);
	
	this._popupElement.appendChild(this._leftDiv);
	this._popupElement.appendChild(this._middleDiv);
	this._popupElement.appendChild(this._rightDiv);
	this._popupElement.appendChild(this._select);
}

DOMPopup.prototype.setValues = function(htmlValues, textValues, values)
{
	this._htmlValues = htmlValues.slice();
	this._textValues = textValues.slice();
	this._values = values.slice();
	
	delete this._valueMap;
	this._valueMap = null;
	this._valueMap = new Object;

	// remove old values
	this._select.options.length = 0;
	
	// create hash to html values
	for (var i = 0; i < values.length; i++) {
		this._valueMap[values[i]] = htmlValues[i];
		this._select.add(new Option(textValues[i], values[i]));
	}
	
	if (values.length > 0) {
		this.value = values[0];
		this._middleDiv.innerHTML = htmlValues[0];
	}
}

DOMPopup.prototype.setEnabled = function(enabled)
{
	this._select.disabled = !enabled;
}

DOMPopup.prototype.setSelectedIndex = function(idx)
{
	if (idx < this._select.options.length) {
		this._select.selectedIndex = idx;
		this._update();
	}
}

DOMPopup.prototype.setValue = function(value)
{
	var options = this._select.options;
	
	for (var i = 0; i < options.length; i++) {
		if (value == options[i].value) {
			this._select.selectedIndex = i;
			this._update();
			
			return;
		}
	}
}

/* private methods */

DOMPopup.prototype._update = function()
{
	this._middleDiv.innerHTML = this._valueMap[this._select.value];
	this.value = this._select.options[this._select.selectedIndex].value;
}

DOMPopup.prototype._onchange = function(event)
{
	this._update();
	
	try {
		if (this.onchange != null) {
			this.onchange(event);
		}
	}
	catch(ex) {
		throw ex;
	}
	finally {
		if (event) {
			event.stopPropagation();
			event.preventDefault();
		}
	}
}
