/*
 * Dine-O-Matic v2.0
 * Copyright 2007 - Iconfactory - http://iconfactory.com
 * 
 * Programmer: Joe Roback <joe.roback@gmail.com>
 * Graphics: Talos Tsui, Corey Marion, David Lanham
 * Audio: Dave Brasgalla
 *
 */

function DOMTabView()
{
	this.enabled = true;
	this._tabs = new Array();
	
	this._init();
}

DOMTabView.prototype._init = function()
{
	// event handlers
	var _self = this;
	this._clickHandler = function(event) { _self._click(event); }
	this._mousedownHandler = function(event) { _self._mousedown(event); }
	this._mousemoveHandler = function(event) { _self._mousemove(event); }
	this._mouseoutHandler = function(event) { _self._mouseout(event); }
	this._mouseoverHandler = function(event) { _self._mouseover(event); }
	this._mouseupHandler = function(event) { _self._mouseup(event); }
}

DOMTabView.prototype._addEventListeners = function(tab)
{
	tab.addEventListener('click', this._clickHandler, true);
	tab.addEventListener('mouseover', this._mouseoverHandler, true);
	tab.addEventListener('mouseout', this._mouseoutHandler, true);
}

DOMTabView.prototype._removeEventListeners = function(tab)
{
	tab.removeEventListener('click', this._clickHandler, true);
	tab.removeEventListener('mouseover', this._mouseoverHandler, true);
	tab.removeEventListener('mouseout', this._mouseoutHandler, true);
}

DOMTabView.prototype.addView = function(content, tab, label, width, height, tabImg, tabSelectedImg, tabHoverImg, selected, onshow)
{
	tab.tabIndex = this._tabs.length;
	this._tabs[this._tabs.length] = [content, tab, tabImg, tabSelectedImg, tabHoverImg, onshow, selected];
	
	var img = new Image(width, height);
	img.src = tabImg;
	img = new Image(width, height);
	img.src = tabSelectedImg;
	img = new Image(width, height);
	img.src = tabHoverImg;
	
	tab.innerText = label;
	tab.style.textAlign = 'center';
	tab.style.verticalAlign = 'middle';
	tab.style.width = width + 'px';
	tab.style.height = height + 'px';
	tab.style.lineHeight = height + 'px';
	tab.style.overflow = 'hidden';
	
	if (selected) {
		content.style.display = 'block';
		tab.style.background = 'url(' + tabSelectedImg + ')';
	}
	else {
		content.style.display = 'none';
		tab.style.background = 'url(' + tabImg + ')';
		
		if (this.enabled) {
			this._addEventListeners(tab);
		}
	}
}

DOMTabView.prototype.remove = function()
{
	var parent = this._container.parentNode;
	parent.removeChild(this._container);
}

DOMTabView.prototype.setEnabled = function(enabled)
{
	var tab = null;
	
	this.enabled = enabled;
	if (enabled) {
		for (var i = 0; i < this._tabs.length; i++) {
			if (!this._tabs[i][6]) {
				this._addEventListeners(this._tabs[i][1]);
			}
		}
	}
	else {
		for (var i = 0; i < this._tabs.length; i++) {
			this._removeEventListeners(this._tabs[i][1]);
		}
	}
}

DOMTabView.prototype._click = function(event)
{
	var content = null;
	var tab = null;
	
	for (var i = 0; i < this._tabs.length; i++) {
		if (i != event.target.tabIndex) {
			this._tabs[i][6] = false;
			content = this._tabs[i][0];
			tab = this._tabs[i][1];
			
			content.style.display = 'none';
			tab.style.background = 'url(' + this._tabs[i][2] + ')';
			this._addEventListeners(tab);
		}
	}

	tab = event.target;
	content = this._tabs[tab.tabIndex][0];
	content.style.display = 'block';
	tab.style.background = 'url(' + this._tabs[tab.tabIndex][3] + ')';
	this._removeEventListeners(tab);
	this._tabs[tab.tabIndex][6] = true;
	
	try {
		// fire off user event handler
		if (this._tabs[tab.tabIndex][5] != null) {
			this._tabs[tab.tabIndex][5](event);
		}
	}
	catch (ex) {
		throw ex;
	}
	finally {
		if (event) {
			event.stopPropagation();
			event.preventDefault();
		}
	}
}

DOMTabView.prototype._mousedown = function(event)
{
	if (event) {
		event.stopPropagation();
		event.preventDefault();
	}
}

DOMTabView.prototype._mouseup = function(event)
{
	if (event) {
		event.stopPropagation();
		event.preventDefault();
	}
}

DOMTabView.prototype._mouseover = function(event)
{
	var tab = event.currentTarget;
	
	if (this._tabs[tab.tabIndex][4] != null) {
		this._tabs[tab.tabIndex][1].style.background = 'url(' + this._tabs[tab.tabIndex][4] + ')';
	}
	
	if (event) {
		event.stopPropagation();
		event.preventDefault();
	}
}

DOMTabView.prototype._mouseout = function(event)
{
	var tab = event.currentTarget;
	this._tabs[tab.tabIndex][1].style.background = 'url(' + this._tabs[tab.tabIndex][2] + ')';

	if (event) {
		event.stopPropagation();
		event.preventDefault();
	}
}

DOMTabView.prototype._mousemove = function(event)
{
	if (event) {
		event.stopPropagation();
		event.preventDefault();
	}
}
